@@ -1,9 +1,12 @@
 use proc_macro::TokenStream;
 use quote::quote;
-use syn::Expr;
-use syn_rsx::{parse, Node, NodeType};
+use syn::{
+    parse::{discouraged::Speculative, ParseBuffer},
+    Expr, ExprLit, Result, Token,
+};
+use syn_rsx::{parse_with_config, Node, NodeType, ParserConfig};
 
-fn walk_nodes(nodes: Vec<Node>) -> (String, Vec<Expr>) {
+fn walk_nodes(nodes: Vec<Node>, nodes_context: Option<NodeType>) -> (String, Vec<Expr>) {
     let mut out = String::new();
     let mut values = vec![];
 
@@ -14,7 +17,8 @@ fn walk_nodes(nodes: Vec<Node>) -> (String, Vec<Expr>) {
                 out.push_str(&format!("<{}", name));
 
                 // attributes
-                let (html_string, attribute_values) = walk_nodes(node.attributes);
+                let (html_string, attribute_values) =
+                    walk_nodes(node.attributes, Some(NodeType::Attribute));
                 out.push_str(&html_string);
                 values.extend(attribute_values);
                 out.push('>');
@@ -27,7 +31,8 @@ fn walk_nodes(nodes: Vec<Node>) -> (String, Vec<Expr>) {
                 }
 
                 // children
-                let (html_string, children_values) = walk_nodes(node.children);
+                let (html_string, children_values) =
+                    walk_nodes(node.children, Some(NodeType::Element));
                 out.push_str(&html_string);
                 values.extend(children_values);
 
@@ -44,11 +49,19 @@ fn walk_nodes(nodes: Vec<Node>) -> (String, Vec<Expr>) {
                 }
             }
             NodeType::Text | NodeType::Block => {
+                if let Some(nodes_context) = &nodes_context {
+                    // If the nodes context is attribute we prefix with whitespace
+                    if nodes_context == &NodeType::Attribute {
+                        out.push(' ');
+                    }
+                }
+
                 out.push_str("{}");
                 values.push(node.value.expect("unexpected missing node value"));
             }
             NodeType::Fragment => {
-                let (html_string, children_values) = walk_nodes(node.children);
+                let (html_string, children_values) =
+                    walk_nodes(node.children, Some(NodeType::Fragment));
                 out.push_str(&html_string);
                 values.extend(children_values);
             }
@@ -70,18 +83,40 @@ fn walk_nodes(nodes: Vec<Node>) -> (String, Vec<Expr>) {
 
 /// Converts HTML to `String`
 ///
-/// Embedded Rust code in braced blocks must return something that implements
-/// [`std::fmt::Display`]. More information about supported syntax can be found [here].
-///
-/// [here]: https://docs.rs/syn-rsx/latest/syn_rsx/#features
+/// This macro should only be used from the `yate` crate, not directly, as
+/// things will break otherwise.
 #[proc_macro]
 pub fn html(tokens: TokenStream) -> TokenStream {
-    match parse(tokens) {
+    match parse_with_config(
+        tokens,
+        ParserConfig::new().transform_block(|input| {
+            let fork = input.fork();
+            if let Ok(text) = parse_escape_syntax(&fork) {
+                input.advance_to(&fork);
+                if !input.is_empty() {
+                    return Err(input.error("escape syntax should end after the last %"));
+                }
+
+                Ok(Some(quote! { yate::html_escape::encode_text(#text) }))
+            } else {
+                Ok(None)
+            }
+        }),
+    ) {
         Ok(nodes) => {
-            let (html_string, values) = walk_nodes(nodes);
+            let (html_string, values) = walk_nodes(nodes, None);
             quote! { format!(#html_string, #(#values),*) }
         }
         Err(error) => error.to_compile_error(),
     }
     .into()
 }
+
+fn parse_escape_syntax(input: &ParseBuffer) -> Result<ExprLit> {
+    input.parse::<Token![%]>()?;
+    input.parse::<Token![=]>()?;
+    let text = input.parse::<ExprLit>()?;
+    input.parse::<Token![%]>()?;
+
+    Ok(text)
+}
